use std::fs::File;
use std::io::BufReader;
use std::path::{Path, PathBuf};
use std::time::Duration;
use rodio::{Decoder, OutputStream, Sink, Source};


//-----------------------------------------Structs


///Stores a list of SongMetas
#[derive(Debug, Clone)]
struct Playlist {
    songs: Vec<SongMeta>,
}

struct AudioPlayer {
    sink: Sink,
    playlist: Vec<SongMeta>,
}


//-----------------------------------Errors
#[derive(Debug, Clone)]
enum PlayerError {
    IndexNotFound,
    SomethingElse,
}


#[derive(Debug, Clone)]
enum PlaylistError {
    IndexNotFound,
    SomethingElse,
}


//-------------------------------------Impls


impl Playlist {
    ///Returns a new empty playlist.
    fn new() -> Self {
        Self {
            songs: vec![],
        }
    }

    ///Returns a playlist from a file containing a list of audiofile paths.
    fn from_file(path: &Path) -> Self;

    ///Removes a song at index (if possible)
    fn pop() -> Result<(), PlayListError>;


}

impl AudioPlayer {

    ///Returns a new empty player.
    fn new() -> Self {
        let (_stream, stream_handle) = OutputStream::try_default().unwrap();
        let sink = Sink::try_new(&stream_handle).unwrap();
        let playlist = vec![];

        Self {
            sink: sink,
            playlist: playlist,
        }
    }

    ///Creates a new player from an existing playlist.
    ///Does not consume the playlist.
    fn from_playlist(playlist: &Playlist) -> Self {
        let (_stream, stream_handle) = OutputStream::try_default().unwrap();
        let sink = Sink::try_new(&stream_handle).unwrap();

        Self {
            sink: sink,
            playlist: playlist.clone(),
        }
    }


    ///Appends a playlist to the current one, consuming it.
    fn add_playlist(&mut self, playlist: Vec<SongMeta>) {
        self.playlist.append(&mut playlist);
    }

    fn skip_forward(&mut self);
    fn skip_backwards(&mut self);
    fn pause(&mut self);
    fn play(&mut self);
    fn set_vol(&mut self, vol: f32);
    fn goto(&mut self, index: usize) -> Result<(), PlayerError>;
}

